#include "stdafx.h"
#include "onipatch.h"

typedef unsigned char	u8;
typedef unsigned short	u16;
typedef unsigned long	u32;

struct PatchEntry
{
	u32	offset;
	u32	length;
	u8	data[0x100];	// just don't make patches bigger than this kthx
};

PatchEntry	kPatchList[] =
{
	{	0x0000BDA8,	0x00000031,
		{	0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 
			0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x8B, 0x0D, 0x00, 0x06, 0x56, 0x00, 0x68, 0x84, 0x10, 
			0x53, 0x00, 0x8B, 0x91, 0x78, 0x05, 0x00, 0x00, 0x52, 0xE8, 0x0A, 0x37, 0x11, 0x00, 0x83, 0xC4, 
			0x20 } },
	{	0x000247F0,	0x00000001,
		{	0xC3 } },
	{	0x00024860,	0x00000001,
		{	0xC3 } },
	{	0x00031C40,	0x00000001,
		{	0xC3 } },
	{	0x0007A0D0,	0x00000001,
		{	0xC3 } },
	{	0xFFFFFFFF }
};

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	OPENFILENAME	openDialogDescriptor;
	char			filePath[8192] = { 0 };

	memset(&openDialogDescriptor, 0, sizeof(openDialogDescriptor));

	openDialogDescriptor.lStructSize =	sizeof(OPENFILENAME);
	openDialogDescriptor.hwndOwner =	NULL;
	openDialogDescriptor.lpstrFile =	filePath;
	openDialogDescriptor.nMaxFile =		sizeof(filePath);
	openDialogDescriptor.lpstrTitle =	"Locate original copy of Oni";
	openDialogDescriptor.Flags =		OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_PATHMUSTEXIST;

	if(GetOpenFileName(&openDialogDescriptor))
	{
		HANDLE	in = CreateFile(openDialogDescriptor.lpstrFile, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if(in)
		{
			u32	fileLength;

			fileLength = GetFileSize(in, NULL);

			u8	* fileData = new u8[fileLength];
			if(fileData)
			{
				u32	bytesTransferred;

				ReadFile(in, fileData, fileLength, &bytesTransferred, NULL);
				CloseHandle(in);
				in = NULL;

				if(fileLength == bytesTransferred)
				{
					u32	checksum = 0;

					for(u32 i = 0; i < fileLength; i++)
					{
						checksum += fileData[i];
						checksum = (checksum << 15) | (checksum >> (32 - 15));
					}

					if(
						(checksum == 0x1e166706) ||
						(MessageBox(
							NULL,
							"Executable checksum failed: you may have a different version of Oni. Continue anyway?",
							"Checksum Failure",
							MB_YESNO | MB_ICONWARNING | MB_DEFBUTTON2) == IDYES))
					{
						for(PatchEntry * patch = kPatchList; patch->offset != 0xFFFFFFFF; patch++)
						{
							if((patch->offset + patch->length) <= fileLength)
							{
								memcpy(&fileData[patch->offset], patch->data, patch->length);
							}
						}

						filePath[0] = 0;

						memset(&openDialogDescriptor, 0, sizeof(openDialogDescriptor));

						openDialogDescriptor.lStructSize =	sizeof(OPENFILENAME);
						openDialogDescriptor.hwndOwner =	NULL;
						openDialogDescriptor.lpstrFile =	filePath;
						openDialogDescriptor.nMaxFile =		sizeof(filePath);
						openDialogDescriptor.lpstrTitle =	"Save patched executable";
						openDialogDescriptor.Flags =		OFN_NOCHANGEDIR | OFN_PATHMUSTEXIST;

						if(GetSaveFileName(&openDialogDescriptor))
						{
							HANDLE	out = CreateFile(openDialogDescriptor.lpstrFile, GENERIC_WRITE, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);

							if(out)
							{
								WriteFile(out, fileData, fileLength, &bytesTransferred, NULL);

								if(fileLength == bytesTransferred)
								{
									MessageBox(NULL, "Patch applied.", "onipatch", MB_OK);
								}
								else
								{
									MessageBox(NULL, "Error writing output file.", "onipatch: error", MB_OK);
								}

								CloseHandle(out);
							}
							else if(GetLastError() == ERROR_ALREADY_EXISTS)
							{
								MessageBox(NULL, "Don't overwrite your original Oni executable; keep it as a backup.", "onipatch: error", MB_OK);
							}
							else
							{
								MessageBox(NULL, "Couldn't open output file.", "onipatch: error", MB_OK);
							}
						}
					}
				}
				else
				{
					MessageBox(NULL, "Error reading input file.", "onipatch: error", MB_OK);
				}
			}
			else
			{
				MessageBox(NULL, "Out of memory.", "onipatch: error", MB_OK);
			}

			if(in)
			{
				CloseHandle(in);
			}
		}
		else
		{
			MessageBox(NULL, "Couldn't open input file.", "onipatch: error", MB_OK);
		}
	}

	return 0;
}
